﻿define([
    // dojo 
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/_base/array",
    "dojo/dom-style",

    // dijit
    "dijit/form/Button",

    // epi-addons 
    "epi-forms/contentediting/editors/ChoiceWithEditor",

    // resources
    "epi/i18n!epi/cms/nls/episerver.campaign.editview",
],
    function (
        // dojo 
        declare,
        lang,
        Deferred,
        array,
        domStyle,

        // dijit
        Button,

        // epi-addons 
        ChoiceWithEditor,

        // resources
        res
    ) {

        // module: 
        //      connectforcampaign/widget/ChoiceWithRecipientAndOptInEditor
        // summary: 
        //      Override to pass additional item params which are used to create RecipientAndOptInSelector item later.
        // tags: 
        //      Override

        return declare([ChoiceWithEditor], {
            // ======================================================================= 
            // Public, overrided stubs 
            // ======================================================================= 

            res: res,

            _itemWidgetsLoadedDefer: null,

            _resetButton: null,

            // =======================================================================
            // Public, overrided stubs
            // =======================================================================

            buildRendering: function () {
                this._itemWidgetsLoadedDefer = new Deferred();

                this.inherited(arguments);

                this._resetButton = new Button({
                    label: res.clearlabel,
                    onClick: lang.hitch(this, function () {
                        this.onFocus(); // need this to make the editor start editing
                        this.set("value", null);
                        this.onChange(this.value);
                        this.onBlur(); // for stop editing and create undo step

                        // Clear the selector combobox value
                        var selectors = this.get("itemWidgets");
                        array.forEach(selectors, lang.hitch(this, function (selector) {
                            if (selector._extendedWidget) {
                                // Set the value of the extended widget (combobox) to empty.
                                // Here we pass _recordFieldSeparator to ensure the value format.
                                selector.set("extendedWidgetValue", { value: this._recordFieldSeparator });
                                selector._calculateSelectorValue && selector._calculateSelectorValue();
                            }
                        }));
                    })
                });

                this._itemWidgetsLoadedDefer.then(lang.hitch(this, function (itemWidgets) {
                    this._resetButton.startup();
                    this._resetButton.placeAt(this.domNode, "last");
                    domStyle.set(this._resetButton.domNode, {
                        marginTop: "15px",
                        "float": "left"
                    });
                }));
            },

            _setupItemWidget: function (/*Object*/itemWidget) {
                // tags:
                //      override

                this.inherited(arguments);

                // Resolve _itemWidgetsLoadedDefer when all selectors have been loaded
                if (this._itemWidgets.length === this.selections.length) {
                    this._itemWidgetsLoadedDefer.resolve(this._itemWidgets);
                }
            },

            postMixInProperties: function () {

                this.inherited(arguments);

                !this.itemParams && (this.itemParams = {});

                lang.mixin(this.itemParams, {
                    campaignDataStoreName: this.campaignDataStoreName,
                    campaignAdaptedConnectorId: this.campaignAdaptedConnectorId,
                    campaignDatasourceFormat: this.campaignDatasourceFormat,
                    marketingConnectorsId: this.marketingConnectorsId
                });
            },

            _buildItemParams: function (/*Object*/item) {
                // summary: 
                //      Override to set extendedWidgetType to connectforcampaign/widget/RecipientAndOptInSelector if the item is Marketing Connectors.
                // tags:
                //      Override

                var itemParams = this.inherited(arguments);

                if (item.value.indexOf(this.marketingConnectorsId) == 0) {
                    lang.mixin(itemParams, {
                        extendedWidgetType: "connectforcampaign/widget/RecipientAndOptInSelector"
                    });
                }

                return itemParams;
            },

            getErrorMessage: function () {
                // summary:
                //      Return message when validation fail.
                // tags:
                //      override

                return res.errormessage;
            },

            _setReadOnlyAttr: function (value) {
                this._resetButton.set("disabled",value);
            }
        });
    });